/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS
* AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY
* REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2019 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : rble_app_rrtp.h
* Version      : 1.00
* Description  : This file is header of the rble_app_rrtp.c
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version Description
*         : 22.02.2019 1.00     First Release
***********************************************************************************************************************/

#ifndef __RBLE_API_RRTP_H__
#define __RBLE_API_RRTP_H__

/* ========================================= Renesas Rapid Transfer Profile ========================================= */

/* Services
 * Services are collections of characteristics and relationship to other services
 * that encapsulate the behavior of part of a device.
*/ 
#define RBLE_SVC_RAPID_TRANSFER {0xe0,0x1b,0x00,0x01,0xba,0x28,0x46,0x6a,0x88,0xfa,0x33,0x77,0x21,0xdc,0x02,0x0b}

/* Characteristics
 * Characteristics are defined attribute types that contain a single logical value.
 */
#define RBLE_CHAR_RRTP_NOTIFY   {0xe0,0x1b,0x00,0x02,0xba,0x28,0x46,0x6a,0x88,0xfa,0x33,0x77,0x21,0xdc,0x02,0x0b}

/* Connection type */
enum RBLE_RRTP_CON_enum
{
    RBLE_RRTP_CON_CFG = 0,
    RBLE_RRTP_CON_NORMAL
};

/* Possible values for setting client configuration characteristics */
enum RBLE_RRTP_CLIENT_CONFIG_enum
{
    RBLE_RRTP_STOP_NTFIND = 0,
    RBLE_RRTP_START_NTF,
    RBLE_RRTP_START_IND
};

/* Internal codes for reading a RRTS characteristic with one single request */
enum RBLE_RRTP_CHAR_CODE_enum
{
    RBLE_RRTP_RRTS_NTF_CFG = 0,     /* Renesas Rapid Transfer Notification Client Config Descriptor */
    RBLE_RRTP_RRTS_IND_CFG,         /* Renesas Rapid Transfer Indication Client Config Descriptor */
};

#define RBLE_RTPC_WRITE_CHAR_MAX    (0x02)

/* Renesas Rqpid Transfer Profile Server Enable Parameter */
typedef struct RBLE_RRTP_SERVER_PARAM_t
{
    uint16_t data_ntf_en;   /* Renesas Rapid Transfer Notification configuration */
} RBLE_RRTP_SERVER_PARAM;

/* Renesas Rqpid Transfer Profile Notify info */
typedef struct RBLE_RRTP_TRANSFER_DATA_t
{
    uint8_t data_len;                   /* data length */
    uint8_t data[RBLE_ATTM_MAX_VALUE];  /* data */
} RBLE_RRTP_TRANSFER_DATA;

/* Structure containing the characteristics handles, value handles and descriptors */
typedef struct RBLE_RRTS_CONTENT_t
{
    uint16_t shdl;
    uint16_t ehdl;

    uint16_t ntf_char_hdl;
    uint16_t ntf_val_hdl;
    uint16_t ntf_cfg_hdl;
    uint8_t  ntf_prop;
    uint8_t  reserved;
} RBLE_RRTS_CONTENT;

/* Renesas Rapid Transfer Profile Call back define */
enum RBLE_RRTP_EVENT_TYPE_enum
{
    RBLE_RRTP_EVENT_SERVER_ENABLE_COMP = 0x01,      /* Server enable Complete Event */
    RBLE_RRTP_EVENT_SERVER_DISABLE_COMP,            /* Server disable Complete Event */
    RBLE_RRTP_EVENT_SERVER_ERROR_IND,               /* Server error Complete Event */
    RBLE_RRTP_EVENT_SERVER_DATA_TRANSFER_COMP,      /* Server data transfer Complete Event */
    RBLE_RRTP_EVENT_SERVER_CHG_INDNTF_IND,          /* Server change Indicate/Notification Cfg Event */
    RBLE_RRTP_EVENT_SERVER_COMMAND_DISALLOWED_IND,  /* Server Command disallwed Complete Event */
    RBLE_RRTP_EVENT_CLIENT_ENABLE_COMP = 0x81,      /* Client enable Complete Event */
    RBLE_RRTP_EVENT_CLIENT_DISABLE_COMP,            /* Client disable Complete Event */
    RBLE_RRTP_EVENT_CLIENT_ERROR_IND,               /* Client error Complete Event */
    RBLE_RRTP_EVENT_CLIENT_NOTIFY,                  /* Client Notify Complete Event */
    RBLE_RRTP_EVENT_CLIENT_INDICATE,                /* Client Indicate Complete Event */
    RBLE_RRTP_EVENT_CLIENT_READ_CHAR_RESPONSE,      /* Client read Characteristic Complete Event */
    RBLE_RRTP_EVENT_CLIENT_WRITE_CHAR_RESPONSE,     /* Client write Characteristic Complete Event */
    RBLE_RRTP_EVENT_CLIENT_COMMAND_DISALLOWED_IND   /* Client Command disallwed Complete Event */
};

typedef uint8_t RBLE_RRTP_EVENT_TYPE;

/* Server */
typedef struct RBLE_RTPS_EVENT_t
{
    RBLE_RRTP_EVENT_TYPE    type;                   /* Event type */
    uint8_t                 reserved;
    union Event_Rtps_Parameter_u
    {
        /* Generic Event */
        RBLE_STATUS     status;

        /* RBLE_EVT_RRTP_Server_Enable_Comp */
        struct RBLE_RRTP_Server_Enable_t
        {
            uint16_t                conhdl;
            RBLE_STATUS             status;
            uint8_t                 reserved;
        } server_enable;

        /* RBLE_EVT_RRTP_Server_Disable_Comp */
        struct RBLE_RRTP_Server_Disable_t
        {
            uint16_t                conhdl;
            RBLE_STATUS             status;
            uint8_t                 reserved;
            RBLE_RRTP_SERVER_PARAM  server_info;
        } server_disable;

        /* RBLE_EVT_RRTP_Server_Error_Ind */
        struct RBLE_RRTP_Server_Error_Ind_t
        {
            uint16_t                conhdl;
            RBLE_STATUS             status;
            uint8_t                 reserved;
        } error_ind;

        /* RBLE_EVT_RRTP_SERVER_SEND_NOTIFY_COMP */
        struct RBLE_RRTP_Server_Data_Transfer_t
        {
            uint16_t                conhdl;
            RBLE_STATUS             status;
            uint8_t                 reserved;
        } data_transfer;
    } param;
} RBLE_RTPS_EVENT;

/* Client */
typedef struct RBLE_RTPC_EVENT_t
{
    RBLE_RRTP_EVENT_TYPE    type;       /* Event Type */
    uint8_t                 reserved;
    union Event_Rtpc_Parameter_u
    {
        /* Generic Event */
        RBLE_STATUS     status;

        /* RBLE_EVT_RRTP_Client_Enable_Comp */
        struct RBLE_RRTP_Client_Enable_t
        {
            uint16_t            conhdl;
            RBLE_STATUS         status;
            uint8_t             reserved;
            RBLE_RRTS_CONTENT   rtps;
        } client_enable;

        /* RBLE_EVT_RRTP_Client_Disable_Comp */
        struct RBLE_RRTP_Client_Disable_t
        {
            uint16_t            conhdl;
            RBLE_STATUS         status;
            uint8_t             reserved;
        } client_disable;

        /* RBLE_EVT_RRTP_Client_Notify_Ind */
        struct RBLE_RRTP_Client_Notify_Ind_t
        {
            uint16_t            conhdl;
            uint8_t             data_len;
            uint8_t             data[RBLE_ATTM_MAX_VALUE];
        } notify;

        /* RBLE_EVT_RRTP_Client_Read_Char_Response */
        struct RBLE_RRTP_Client_Read_Char_Response_t
        {
            uint16_t            conhdl;
            uint8_t             att_code;
            RBLE_ATT_INFO_DATA  data;
        } rd_char_resp;

        /* RBLE_EVT_RRTP_Client_Write_Char_Response */
        struct RBLE_RRTP_Client_Write_Char_Response_t
        {
            uint16_t            conhdl;
            uint8_t             att_code;
        } wr_char_resp;
    } param;
} RBLE_RTPC_EVENT;

typedef void (*RBLE_RTPS_EVENT_HANDLER)(RBLE_RTPS_EVENT * event);   /* RRT(Server) Event CALL BACK Declaration */
typedef void (*RBLE_RTPC_EVENT_HANDLER)(RBLE_RTPC_EVENT * event);   /* RRT(Client) Event CALL BACK Declaration */

/* ----------------------------------------- Renesas Rapit Transfer Profile ------------------------------------------*/
RBLE_STATUS RBLE_RRTP_Server_Enable(uint16_t conhdl, uint8_t con_type, RBLE_RRTP_SERVER_PARAM *param, RBLE_RTPS_EVENT_HANDLER call_back);
RBLE_STATUS RBLE_RRTP_Server_Disable(uint16_t conhdl);
RBLE_STATUS RBLE_RRTP_Server_Data_Transfer(uint16_t conhdl, RBLE_RRTP_TRANSFER_DATA * notify_info, uint16_t Notify_No);
RBLE_STATUS RBLE_RRTP_Client_Enable(uint16_t conhdl, uint8_t con_type, RBLE_RRTS_CONTENT *scs, RBLE_RTPC_EVENT_HANDLER call_back);
RBLE_STATUS RBLE_RRTP_Client_Disable(uint16_t conhdl);
RBLE_STATUS RBLE_RRTP_Client_Read_Char(uint16_t conhdl, uint8_t char_code);
RBLE_STATUS RBLE_RRTP_Client_Write_Char(uint16_t conhdl, uint8_t char_code, uint8_t *write_value);

#endif /* __RBLE_API_RRTP_H__ */

