/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS
* AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY
* REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2019 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : rble_app_rrtpc.c
* Version      : 1.00
* Description  : This file is rBLE Application.
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version Description
*         : 22.02.2019 1.00     First Release
***********************************************************************************************************************/
#include "rwble_config.h"
#include "arch.h"

#include    <stdio.h>
#include    <string.h>
#include    <stdlib.h>

#include    "rble.h"
#include    "rble_app.h"
#include    "menu_sel.h"
#include    "console.h"

#include    "prf_sel.h"
#include    "rble_api_rrtp.h"
#include    "rble_app_rrtpc.h"

/******************************** Define Declaration **********************************/
#if (PRF_SEL_RTPC) /* __ENABLE_USE_RRTPC__ */

RBLE_RRTP_INFO    rrtp_info = {
    FALSE,      /* notify enable flag */
    FALSE,      /* indicate enable flag */
    FALSE,      /* timer interrupt enable flag */
    1,          /* timer interval time(1*10ms) */
    20,         /* notify data length */
    20          /* indicate data length */
};

/********************************* PRIVATE PROTOTYPE **********************************/
extern BOOL RBLE_GAP_Reset_Test(void);
static void RBLE_APP_RTPC_CallBack(RBLE_RTPC_EVENT * event);
static void RTP_Event_Disp(char_t *event_str, RBLE_STATUS status);
static void RBLEL_Notify_Disp(RBLE_RTPC_EVENT *event);
BOOL RBLE_RTP_Client_Write_Char_Test(void);
void RBLEL_Throughput_Disp(void);

/* ------------------------- Renesas Rapid Transfer Profile --------------------------*/

/***************************** PUBLIC Memory Declaration ******************************/
extern RBLE_RRTP_INFO rrtp_info;

extern uint8_t Role_Status;
uint8_t     RBLE_Test_Data = 0;
uint8_t     RBLE_Test_Start_Flg = false;
int_t       RBLE_Test_Type = -1;
uint16_t    RBLE_Test_Interval_No = 0;
uint8_t     RBLE_Test_Notify_en = false;
uint16_t    RBLEL_Throughput_Cnt;
uint8_t     notify_count = 0;
uint32_t send_seq_num = 0;
uint32_t send_data_set = 0;
uint8_t send_packet_start = false;
uint32_t send_data_packet_num = 0;

static RBLE_RRTS_CONTENT g_rrtp_svc = { 0 };

#define             DEVICE_SEARCH_MAX               20

extern RBLE_BD_ADDR Remote_Device;
RBLE_BD_ADDR        Device_Search_Result[ DEVICE_SEARCH_MAX ];
uint16_t            Device_Search_Cnt;

uint32_t packet_count = 0;
uint32_t receive_seq_num = 0;

/***************************** PRIVATE Memory Declaration *****************************/

/* --------------------- Renesas Rapid Transfer Server Profile ---------------------- */

/***********************************************************************************************************************
* Function Name: RBLE_APP_RTPC_CallBack
* Description  :
*
* Arguments    : (RBLE_RTPS_EVENT) * event -
* Return Value : None
***********************************************************************************************************************/
static void RBLE_APP_RTPC_CallBack(RBLE_RTPC_EVENT * event)
{

    if ( false == RBLE_Test_Start_Flg ) {
    Console_SetTextAttribute( EVENT_COLOR );
    }

    switch( event->type){
        case RBLE_RRTP_EVENT_CLIENT_ENABLE_COMP:
            RTP_Event_Disp("CLIENT_ENABLE_COMP", event->param.client_enable.status);
            if ( 1 == RBLE_Test_Type ) {
            	RBLE_Parameter_Cnt = 2;
                RBLE_Parameter_Arg[ 0 ] = 0; /* RBLE_RTP_SCS_NTF_CFG (char_code) */
                RBLE_Parameter_Arg[ 1 ] = 1; /* write_val */
                RBLE_RTP_Client_Write_Char_Test();
                RBLE_Parameter_Cnt = 0;
                RBLE_Parameter_Arg[ 0 ] = 0;
                RBLE_Parameter_Arg[ 1 ] = 0;
            } else if ( 2 == RBLE_Test_Type ) {
                RBLE_Parameter_Cnt = 2;
                RBLE_Parameter_Arg[ 0 ] = 1; /* RBLE_RTP_SCS_IND_CFG (char_code) */
                RBLE_Parameter_Arg[ 1 ] = 2; /* write_val */
                RBLE_RTP_Client_Write_Char_Test();
                RBLE_Parameter_Cnt = 0;
                RBLE_Parameter_Arg[ 0 ] = 0;
                RBLE_Parameter_Arg[ 1 ] = 0;
            }
            break;
        case RBLE_RRTP_EVENT_CLIENT_DISABLE_COMP:
            RTP_Event_Disp("CLIENT_DISABLE_COMP", event->param.client_disable.status);
            printf( "Connect Handle = %d\n", event->param.client_disable.conhdl );
            break;
        case RBLE_RRTP_EVENT_CLIENT_NOTIFY:

            RBLE_Test_Start_Flg = false;
            send_packet_start = false;
            RBLEL_Notify_Disp( event );
{
#ifdef  __ERROR_CHECK__
            if ( RBLE_Test_Data != event->param.notify.data[0] ) {
                RBLE_Test_Start_Flg = false;
                printf( "Error(%x)!=(%x)\n", RBLE_Test_Data, event->param.notify.data[0] );
                RBLE_Test_Start_Flg = true;
            }
#endif
            {
/*		uint8_t i; */
            	RBLE_Test_Start_Flg = false;
		/* If output "BYTE data", bitrate is down!! */
		if ( 0 == RBLEL_Throughput_Cnt ) {
                    Console_Set_Timer( 100 ); /* 100 * 10ms => 1S */
                }
		RBLEL_Throughput_Cnt+=event->param.notify.data_len;
            	RBLE_Test_Start_Flg = true;
            }
}
            break;
        case RBLE_RRTP_EVENT_CLIENT_READ_CHAR_RESPONSE:
            RTP_Event_Disp("CLIENT_READ_CHAR_RES", event->param.rd_char_resp.att_code);
            printf( "Connect Handle = %d\n", event->param.rd_char_resp.conhdl );
            ATT_Info_Disp( &event->param.rd_char_resp.data );
            break;
        case RBLE_RRTP_EVENT_CLIENT_WRITE_CHAR_RESPONSE:
            RTP_Event_Disp("CLIENT_WRITE_CHAR_RES", event->param.wr_char_resp.att_code);
            printf( "Connect Handle = %d\n", event->param.wr_char_resp.conhdl );

            if ( (1 == RBLE_Test_Type) || (2 == RBLE_Test_Type) ) {
                RBLE_Test_Start_Flg = true;
                if ( 0 == RBLEL_Throughput_Cnt ) {
                    Console_Send_Timer_msg( RBLEL_Throughput_Disp );
                }
            }

            break;
        case RBLE_RRTP_EVENT_CLIENT_ERROR_IND:
        case RBLE_RRTP_EVENT_CLIENT_COMMAND_DISALLOWED_IND:
        default:
            break;
    }
    if ( false == RBLE_Test_Start_Flg ) {
    Prompt_Disp();
    }
}

/***********************************************************************************************************************
* Function Name: RBLE_RTP_Client_Enable_Test
* Description  : This task is performed by the RBLE_RTP_Client_Enable command.
*
* Arguments    : None
* Return Value : (RBLE_STATUS) Ret_Status -
*                    - TRUE  :01h - Menu Continue
*                    - FALSE :00h - status error
***********************************************************************************************************************/
BOOL RBLE_RTP_Client_Enable_Test(void)
{
    /* ConType */
    uint8_t con_type = RBLE_PRF_CON_DISCOVERY;
    RBLE_STATUS Ret_Status;

    /* Valid Connection Handle ? */
    if ( ILLEGAL_CONNECTION_HANDLE == Connection_Handle ) {
        Illegal_Connection_Print();
    } else {
        if (0x0000 != g_rrtp_svc.shdl) {
            con_type = RBLE_PRF_CON_NORMAL;
        }

        /* API Call */
        Ret_Status = RBLE_RRTP_Client_Enable( Connection_Handle, con_type, &g_rrtp_svc, RBLE_APP_RTPC_CallBack );

        RBLE_Status_Disp( Ret_Status );
    }
    return( TRUE );
}


/***********************************************************************************************************************
* Function Name: RBLE_RTP_Client_Disable_Test
* Description  : This task is performed by the RBLE_RTP_Client_Disable command.
*
* Arguments    : None
* Return Value : (RBLE_STATUS) Ret_Status -
*                    - TRUE  :01h - Menu Continue
*                    - FALSE :00h - status error
***********************************************************************************************************************/
BOOL RBLE_RTP_Client_Disable_Test(void)
{
    RBLE_STATUS Ret_Status;

    /* Valid Connection Handle ? */
    if ( ILLEGAL_CONNECTION_HANDLE == Connection_Handle ) {
        Illegal_Connection_Print();
    } else {

        /* API Call */
        Ret_Status = RBLE_RRTP_Client_Disable( Connection_Handle );

        RBLE_Status_Disp( Ret_Status );
    }
    return( TRUE );
}

/******************************************************************************************/
/**
 *  @brief A RTP_Client_Write_Char command is executed. 
 *
 *
 *  @attention
 *  
 *  @b Contents of management:
 *
 *  @param[in]
 *
 *  @retval TRUE     Menu Continue
 */
/******************************************************************************************/
BOOL RBLE_RTP_Client_Write_Char_Test(void)
{
    RBLE_STATUS                 Ret_Status;
    uint8_t                     write_value[2] = {0};
    uint8_t                     char_code;

	RBLE_Parameter_Arg[ 1 ] = 1;
	
    /* Valid Connection Handle ? */
    if ( ILLEGAL_CONNECTION_HANDLE == Connection_Handle ) {
        Illegal_Connection_Print();
    } else {
        if ( 0 != RBLE_Parameter_Cnt ) {

            switch (RBLE_Parameter_Arg[ 0 ]) {
            case 0:
            default:
                char_code = RBLE_RRTP_RRTS_NTF_CFG;
                break;
            }
            if (RBLE_Parameter_Cnt > 1) {
                write_value[0] = (uint8_t)(RBLE_Parameter_Arg[ 1 ] & 0xff);
                write_value[1] = (uint8_t)((RBLE_Parameter_Arg[ 1 ] >> 8) & 0xff);
            }
        } else {
            char_code = RBLE_RRTP_RRTS_NTF_CFG;
        }

        /* API Call */
        Ret_Status = RBLE_RRTP_Client_Write_Char( Connection_Handle, char_code, &write_value[0] );
        printf( " char_code = %d, write_val = 0x%02X%02X\n", char_code, write_value[1], write_value[0] );

        RBLE_Status_Disp( Ret_Status );
    }
    return( TRUE );
}

/******************************************************************************************/
/**
 *  @brief rBLE RTP Event Disp
 *
 *
 *  @attention
 *  
 *  @b Contents of management:
 *
 *  @param[in] *event_str      event strings
 *  @param[in] status          status
 *
 *  @retval   :void
 */
/******************************************************************************************/
static void RTP_Event_Disp(char_t *event_str, RBLE_STATUS status)
{
    printf( "\nrBLE RRTP EVENT (%s) Status(%s)\n", event_str, Get_Status_Str( status ) );
}

/******************************************************************************************/
/**
 *  @brief RBLEL_Throughput_Disp
 *
 *
 *  @attention
 *  
 *  @b Contents of management:
 *
 *  @param[in] *event_str      event strings
 *  @param[in] status          status
 *
 *  @retval   : void
 */
/******************************************************************************************/
void RBLEL_Throughput_Disp(void)
{

	if ( 0 == RBLE_Test_Type ) {
		RBLE_Test_Start_Flg = false;
		printf("Data[1000] set. send_data_packet_exec_num:%d\n",send_data_packet_num);
		RBLE_Test_Start_Flg = true;
		send_packet_start = true;
		send_data_packet_num = 0;
		packet_count = 0;
		send_seq_num = 0;
		Console_Set_Timer( 100 ); /* 100 * 10ms => 1S */
	}
	/* Original */
	else {
		RBLE_Test_Start_Flg = false;
		printf( "Throughput(b/s) -> %ld\n", ( ( unsigned long )RBLEL_Throughput_Cnt * 8l ));
		RBLE_Test_Start_Flg = true;
		RBLEL_Throughput_Cnt = 0;
	}
}


/******************************************************************************************/
/**
 *  @brief RBLEL_Throughput_Disp
 *
 *
 *  @attention
 *
 *  @retval TRUE     Menu Continue
 */
/******************************************************************************************/
BOOL RBLE_Test_Select(void)
{
    bool    Flg = false;

    if ( 0 != RBLE_Parameter_Cnt ) {
        if ( (1 == RBLE_Parameter_Arg[ 0 ]) || (2 == RBLE_Parameter_Arg[ 0 ]) ) {
            if ( 2 == RBLE_Parameter_Cnt ) {
                if ( 0 != RBLE_Parameter_Arg[ 1 ] ) {
                    if ( 0 != Device_Search_Cnt ) {
                        if ( Device_Search_Cnt >= RBLE_Parameter_Arg[ 1 ] ) {
                            Flg = true;
                        }
                    }
                }
            }
        } else  if ( 0 == RBLE_Parameter_Arg[ 0 ] ) {
            if ( 2 == RBLE_Parameter_Cnt ) {
                if ( 10 >= RBLE_Parameter_Arg[ 1 ] ) {
                    Flg = true;
                }
            }
        }
    }
    if ( true == Flg ) {
        RBLE_Test_Type = ( int_t )RBLE_Parameter_Arg[ 0 ];
        RBLE_Parameter_Cnt = 0;
        if ( 0 == RBLE_Test_Type ) {
            RBLE_Test_Interval_No = (uint16_t)RBLE_Parameter_Arg[ 1 ];
        } else if ( (1 == RBLE_Test_Type) || (2 == RBLE_Test_Type) ) {
            Remote_Device = Device_Search_Result[ RBLE_Parameter_Arg[ 1 ] - 1 ];
        }
        RBLE_GAP_Reset_Test();
    } else {
        printf( "TestType 0:Slave Select No(7.5ms + No * 1.25ms), 1:Master Select No(Allow Notification), 2:Master Select No(Allow Indication)\n" );
    }
    return( TRUE );

}

/******************************************************************************************/
/**
 *  @brief RBLEL_Notify_Disp
 *
 *
 *  @attention
 *
 *  @param[in] *event     
 *
 *  @retval   : void
 */
/******************************************************************************************/
static void RBLEL_Notify_Disp(RBLE_RTPC_EVENT *event)
{
    uint8_t i,length;

    length = event->param.notify.data_len;
    printf("Notify [len = %d] = ",length);
    for(i = 0;i<length;i++){
       printf("%02X",event->param.notify.data[i]);
    }
    printf("\n");
}

#endif /* (PRF_SEL_RTPC) __ENABLE_USE_RRTPC__ */

