/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS
* AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY
* REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : r_rrtp.c
* Version      : 1.00
* Description  : This file has a description of the renesas rapid transfer profile.
***********************************************************************************************************************/
/***********************************************************************************************************************
* History : DD.MM.YYYY Version  Description
*         : 09.11.2018 1.00     First Release
***********************************************************************************************************************/


/***********************************************************************************************************************
Includes   <System Includes> , "Project Includes"
***********************************************************************************************************************/
#include <stdio.h>
#include <string.h>

#include "ll.h"

#include "r_rrtp.h"
#include "rble.h"

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/

/***********************************************************************************************************************
Macro definitions
***********************************************************************************************************************/

/***********************************************************************************************************************
Private global variables and functions
***********************************************************************************************************************/

/***********************************************************************************************************************
Export global variables (to be accessed by other files)
***********************************************************************************************************************/
st_rapid_data_t g_rapid_transfer_queue;

/***********************************************************************************************************************
* Function Name: R_RRTP_DataEnqueue
* Description  : This task checks the data for rapid transfer.
*
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
uint8_t R_RRTP_DataEnqueue(uint8_t * data, uint8_t len)
{
    uint8_t ret;

    ret = FALSE;

    if (g_rapid_transfer_queue.in_use < D_RAPID_TRANSFER_QUEUE_SIZE)
    {
        if ((NULL != data) && (len > 0) && (len <= D_RAPID_TRANSFER_BUFFER_SIZE))
        {
            /* Copy the transfer data from pointer to array, and clear pointer. */
            memcpy((void *)&g_rapid_transfer_queue.queue_array[g_rapid_transfer_queue.push_i].data[0],
                    (void *)data, len);
            g_rapid_transfer_queue.queue_array[g_rapid_transfer_queue.push_i].data_len = len;

            /* Update queue information */
            g_rapid_transfer_queue.push_i = (g_rapid_transfer_queue.push_i + 1) % D_RAPID_TRANSFER_QUEUE_SIZE;
            g_rapid_transfer_queue.in_use++;

            ret = TRUE;
        }
        else
        {
            /* Do nothing */
            __nop();
        }
    }
    else
    {
        /* Do nothing */
        __nop();
    }

    return ret;
}

/* End of function R_RRTP_DataEnqueue */

