/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS
* AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY
* REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : rble_app_common.h
* Version      : 1.00
* Device(s)    : R5F11CCC
* Description  : This file is a header file for "rble_app_common.c".
***********************************************************************************************************************/
/***********************************************************************************************************************
* History      : DD.MM.YYYY    Version    Description
*              : DD.MM.2016    1.00       First edition
***********************************************************************************************************************/
#ifndef _rBLE_APP_COMMON_H
#define _rBLE_APP_COMMON_H

#include   "rble_api.h"

/********************************* Define Declaration *********************************/

#define __ENABLE_USE_RRTP__     /* USE Renesas Rapid Transfer Profile */

enum RBLE_APP_KEY_FLAG_enum
{
    RBLE_APP_LTK,
    RBLE_APP_ENCKEY,
    RBLE_APP_IDKEY,
    RBLE_APP_SIGNKEY
};

enum RBLE_APP_PROFILE_enum
{
#ifdef __ENABLE_USE_GLP__
    RBLE_APP_PROFILE_GLP,
#endif /* __ENABLE_USE_GLP__ */
#ifdef __ENABLE_USE_BLP__
    RBLE_APP_PROFILE_BLP,
#endif /* __ENABLE_USE_BLP__ */
    RBLE_APP_PROFILE_RRTP
};

/* rble app status */
typedef enum
{
    RBLE_APP_STATUS_NONE = 0x00,
    RBLE_APP_STATUS_INIT_AP_REQ,
    RBLE_APP_STATUS_RF_EFFECTIVE_REQ,
    RBLE_APP_STATUS_RF_DISABLE_REQ,
    RBLE_APP_STATUS_ADVERTISE_START,
    RBLE_APP_STATUS_ADVERTISE_STOP,
    RBLE_APP_STATUS_CONNECT,
    RBLE_APP_STATUS_DISCONNECT,
    RBLE_APP_STATUS_TIME_REQ,
    RBLE_APP_STATUS_DATA_ID_LIST_REQ,
    RBLE_APP_STATUS_DATA_REQ,
    RBLE_APP_STATUS_DATA_SAVE_REQ,
    RBLE_APP_STATUS_DATA_DELETE_REQ,
    RBLE_APP_STATUS_DATA_TRANSFER_REQ,
    RBLE_APP_STATUS_COMMAND_CANCEL_REQ,
    RBLE_APP_STATUS_TIME_SYNC,
    RBLE_APP_STATUS_RECORD_REPORT_SENSOR,
    RBLE_APP_STATUS_RECORD_REPORT_RF,
    RBLE_APP_STATUS_RECORD_DELETE_SENSOR,
    RBLE_APP_STATUS_RECORD_DELETE_RF,
    RBLE_APP_STATUS_RECORD_NUM_REPORT_SENSOR,
    RBLE_APP_STATUS_RECORD_NUM_REPORT_RF,
    RBLE_APP_STATUS_RECORD_CANCEL,
    RBLE_APP_STATUS_RBLE_STATUS_REQ,
    RBLE_APP_STATUS_PRESSURE_DATA_REQ,
    RBLE_APP_STATUS_MEASUREMENT_START,
    RBLE_APP_STATUS_MEASUREMENT_STOP,
    RBLE_APP_STATUS_MEASUREMENT_SAFETY,
    RBLE_APP_STATUS_GET_CUFF_THRESHOLD_REQ,
    RBLE_APP_STATUS_SET_CUFF_THRESHOLD_REQ
} RBLE_APP_STATUS;

typedef enum 
{
    RBLE_RF_UNINITIALIZED,
    RBLE_RF_INITIALIZED,
    RBLE_RF_ENABLED,
    RBLE_RF_ADVERTISING,
    RBLE_RF_CONNECTED,
    RBLE_RF_SERVER_ENABLED,
} RBLE_RF_STATUS;

typedef struct
{
    BOOL                            Key_Set_Flags;
    struct RBLE_EVT_SM_Key_t        EncKey;
    struct RBLE_EVT_SM_Key_t        IrKey;
    struct RBLE_EVT_SM_Key_t        SignKey;
} RBLE_APP_KEY_DATA;

/***************************** PUBLIC Memory Declaration ******************************/

extern uint16_t g_rble_rf_status;

/******************************* PROTOTYPE Declaration ********************************/

bool RBLE_App_InitializeAPIRequested(st_init_api_info_t * init_api_info_t);
bool RBLE_App_EnableRfRequested(bool startAdvertise);
bool RBLE_App_DisableRfRequested(void);
bool RBLE_App_StartAdvertiseRequested(void);
bool RBLE_App_StopAdvertiseRequested(void);
void RBLE_App_DisconnectedRequested(void);
bool RBLE_App_GainTimeRequested(void);

bool RBLE_App_CancelRequested(void);

bool RBLE_App_Init(void);
bool RBLE_App_StatusRequested(void);

void RBLE_App_SendEvent_Com(RBLE_APP_STATUS status, uint8_t result);

void RBLE_App_TimeSynchronized(RBLE_TIPC_EVENT * event);

RBLE_STATUS TIP_Client_Enable(void);

#endif  /* _rBLE_APP_COMMON_H */

