/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS
* AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY
* REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2019 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : rble_app_rrtp.c
* Version      : 1.00
* Description  : This file is rBLE Application.
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version Description
*         : DD.MM.2019 1.00     First Release
***********************************************************************************************************************/
#include "arch.h"

#include "rble_api.h"
#include "rble_external.h"
#include "rble_app.h"
#include "rble_app_common.h"
#include "rble_app_rrtp.h"
#include "rble_api_rrtp.h"
#include "r_rrtp.h"

#include "prf_sel.h"

/******************************** Define Declaration **********************************/

#if (PRF_SEL_RTPS) /* __ENABLE_USE_RRTP__ */

RBLE_RRTP_INFO    rrtp_info = {
    FALSE,      /* notify enable flag */
    FALSE,      /* indicate enable flag */
    FALSE,      /* timer interrupt enable flag */
    1,          /* timer interval time(1*10ms) */
    20,         /* notify data length */
    20          /* indicate data length */
};

/********************************* PRIVATE PROTOTYPE **********************************/

/* ------------------------- Renesas Rapid Transfer Profile --------------------------*/

/***************************** PUBLIC Memory Declaration ******************************/
extern RBLE_RRTP_INFO rrtp_info;


/* RBLE APP STATUS */
extern RBLE_APP_STATUS      rbleAppStatus;

/***************************** PRIVATE Memory Declaration *****************************/
volatile uint8_t g_rrtp_moredata;

uint16_t         more_data_count;

/* --------------------- Renesas Rapid Transfer Server Profile ---------------------- */

/***********************************************************************************************************************
* Function Name: RBLE_APP_RRTP_CallBack
* Description  :
*
* Arguments    : (RBLE_RTPS_EVENT) * event -
* Return Value : None
***********************************************************************************************************************/
void RBLE_APP_RRTP_CallBack(RBLE_RTPS_EVENT * event)
{
    switch (event->type)
    {
        case RBLE_RRTP_EVENT_SERVER_ENABLE_COMP:
            {
                /* If status is connect */
                if (RBLE_APP_STATUS_CONNECT == rbleAppStatus)
                {
                    if (RBLE_OK == event->param.status)
                    {
                        rbleAppStatus = RBLE_APP_STATUS_NONE;
                    }
                    else
                    {
                        /* Do nothing */
                        __nop();
                    }
                }
                else
                {
                    /* Do nothing */
                    __nop();
                }
            }
            break;
        case RBLE_RRTP_EVENT_SERVER_DISABLE_COMP:
            {
                /* Do nothing */
                __nop();
            }
            break;
        case RBLE_RRTP_EVENT_SERVER_ERROR_IND:
            {
                /* Do nothing */
                __nop();
            }
            break;
        case RBLE_RRTP_EVENT_SERVER_DATA_TRANSFER_COMP:
            {
                /* Do nothing */
                __nop();
            }
            break;
        case RBLE_RRTP_EVENT_SERVER_CHG_INDNTF_IND:
            {
                /* update info */
                if (RBLE_RRTP_RRTS_NTF_CFG == event->param.cfg_indntf.char_code)
                {
                    if (event->param.cfg_indntf.cfg_val & RBLE_RRTP_START_NTF)
                    {
                        rrtp_info.Notify_en = TRUE;
                    }
                    else
                    {
                        rrtp_info.Notify_en = FALSE;
                    }
                    g_rble_rf_status = RBLE_RF_SERVER_ENABLED;
                }
            }
            break;
        case RBLE_RRTP_EVENT_SERVER_COMMAND_DISALLOWED_IND:
            {
                /* Do nothing */
                __nop();
            }
            break;
        default:
            {
                /* Do nothing */
                __nop();
            }
            break;
    }
}

/***********************************************************************************************************************
* Function Name: RBLE_App_RrtpServerEnable
* Description  : This task is performed by the RBLE_App_RrtpServerEnable command.
*
* Arguments    : None
* Return Value : (RBLE_STATUS) Ret_Status -
*                    - TRUE  :01h - Menu Continue
*                    - FALSE :00h - status error
***********************************************************************************************************************/
RBLE_STATUS RBLE_App_RrtpServerEnable(void)
{
    RBLE_STATUS                  Ret_Status;
    static uint8_t               con_type = RBLE_RRTP_CON_CFG;
    static RBLE_RRTP_SERVER_PARAM param    = {
        RBLE_RRTP_START_NTF,
    };

    /* Valid Connection Handle ? */
    if (ILLEGAL_CONNECTION_HANDLE == Connection_Handle)
    {
        /* Do nothing */
        __nop();
    }
    else
    {
        /* API Call */
        Ret_Status = RBLE_RRTP_Server_Enable(Connection_Handle, con_type, &param, RBLE_APP_RRTP_CallBack);

    }

    return(Ret_Status);
}

/***********************************************************************************************************************
* Function Name: RBLE_App_RapidTransferHandler
* Description  :
*
* Arguments    :
* Return Value :
***********************************************************************************************************************/
void RBLE_App_RapidTransferHandler(void)
{
    RBLE_STATUS result;

    if (TRUE == rrtp_info.Notify_en)
    {
        while ((more_data_count < g_rrtp_moredata) && (g_rapid_transfer_queue.in_use > 0))
        {
#if 0
            /* Enable this line for more_data number confirmation. */
            g_rapid_transfer_queue.queue_array[g_rapid_transfer_queue.pop_i].data[1] = more_data_count;
#endif

            result = RBLE_RRTP_Server_Data_Transfer(Connection_Handle,
                    (RBLE_RRTP_TRANSFER_DATA *)&g_rapid_transfer_queue.queue_array[g_rapid_transfer_queue.pop_i],
                    more_data_count);

            if (RBLE_OK == result) 
            {
                g_rapid_transfer_queue.in_use--;
                g_rapid_transfer_queue.pop_i = (g_rapid_transfer_queue.pop_i + 1) % D_RAPID_TRANSFER_QUEUE_SIZE;

                more_data_count++;
            }
            else
            {
                break;
            }
        }
    }
    else
    {
        /* Do nothing */
        __nop();
    }
}

#endif /* (PRF_SEL_RTPS) __ENABLE_USE_RRTP__ */

